@@ -21,6 +21,7 @@ use overload (
 
 # ATTRIBUTES
 
+attr 'expects';
 attr 'message';
 attr 'name';
 
@@ -38,10 +39,11 @@ sub build_self {
   my ($self, $data) = @_;
 
   my $name = 'Unknown';
-  my $message = 'Type assertion (%s) failed: received (%s)';
+  my $message = 'Type assertion (%s) failed: received (%s), expected (%s)';
 
   $self->name($name) if !$self->name;
   $self->message($message) if !$self->message;
+  $self->expects([]) if !$self->expects;
   $self->conditions;
 
   return $self;
@@ -79,6 +81,12 @@ sub array {
   return $self;
 }
 
+sub arrayref {
+  my ($self, @code) = @_;
+
+  return $self->array(@code);
+}
+
 sub assertion {
   my ($self) = @_;
 
@@ -124,6 +132,12 @@ sub code {
   return $self;
 }
 
+sub coderef {
+  my ($self, @code) = @_;
+
+  return $self->code(@code);
+}
+
 sub coerce {
   my ($self, $data) = @_;
 
@@ -205,6 +219,27 @@ sub enum {
   return $self;
 }
 
+sub expression {
+  my ($self, $data) = @_;
+
+  return $self if !$data;
+
+  my @data = split /\s*\|\s*(?![^\[]*\])/, $data;
+
+  push @{$self->expects}, @data;
+
+  for my $item (@data) {
+    if (my @nested = $item =~ /^((?:array|hash)(?:ref)?)\[([^\]]+)\]$/) {
+      $self->within($nested[0])->expression($nested[1]);
+    }
+    else {
+      $self->accept($item);
+    }
+  }
+
+  return $self;
+}
+
 sub float {
   my ($self, @code) = @_;
 
@@ -269,6 +304,12 @@ sub hash {
   return $self;
 }
 
+sub hashref {
+  my ($self, @code) = @_;
+
+  return $self->hash(@code);
+}
+
 sub identity {
   my ($self, $name) = @_;
 
@@ -316,6 +357,50 @@ sub package {
   return $self;
 }
 
+sub received {
+  my ($self, $data) = @_;
+
+  require Scalar::Util;
+
+  if (!defined $data) {
+    return '';
+  }
+
+  my $blessed = Scalar::Util::blessed($data);
+  my $isvenus = $blessed && $data->isa('Venus::Core') && $data->can('does');
+
+  if (!$blessed && !ref $data) {
+    return $data;
+  }
+  if ($blessed && ref($data) eq 'Regexp') {
+    return "$data";
+  }
+  if ($isvenus && $data->does('Venus::Role::Explainable')) {
+    return $self->dump(sub{$data->explain});
+  }
+  if ($isvenus && $data->does('Venus::Role::Valuable')) {
+    return $self->dump(sub{$data->value});
+  }
+  if ($isvenus && $data->does('Venus::Role::Dumpable')) {
+    return $data->dump;
+  }
+  if ($blessed && overload::Method($data, '""')) {
+    return "$data";
+  }
+  if ($blessed && $data->can('as_string')) {
+    return $data->as_string;
+  }
+  if ($blessed && $data->can('to_string')) {
+    return $data->to_string;
+  }
+  if ($blessed && $data->isa('Venus::Kind')) {
+    return $data->stringified;
+  }
+  else {
+    return $self->dump(sub{$data});
+  }
+}
+
 sub reference {
   my ($self, @code) = @_;
 
@@ -353,6 +438,12 @@ sub scalar {
   return $self;
 }
 
+sub scalarref {
+  my ($self, @code) = @_;
+
+  return $self->scalar(@code);
+}
+
 sub string {
   my ($self, @code) = @_;
 
@@ -395,18 +486,15 @@ sub validate {
 
   require Scalar::Util;
 
-  my $received = defined $data
-    ? (
-    ref $data
-    ? "$data"
-    : (Scalar::Util::looks_like_number($data) ? $data : "'$data'"))
-    : "undef";
+  my $identity = lc(Venus::Type->new(value => $data)->identify);
+  my $expected = (join ' OR ', @{$self->expects}) || 'indescribable constraints';
+  my $message = sprintf($self->message, $self->name, $identity, $expected);
 
   my $throw;
   $throw = $self->throw;
   $throw->name('on.validate');
-  $throw->message(sprintf($self->message, $self->name, $received));
-  $throw->stash(identity => lc(Venus::Type->new(value => $data)->identify));
+  $throw->message($message . "\n\nReceived:\n\n\"@{[$self->received($data)]}\"\n\n");
+  $throw->stash(identity => $identity);
   $throw->stash(variable => $data);
   $throw->error;
 
@@ -439,14 +527,14 @@ sub within {
 
   my $where = $self->new;
 
-  if (lc($type) eq 'hash') {
+  if (lc($type) eq 'hash' || lc($type) eq 'hashref') {
     $self->defined(sub{
       my $value = $_->value;
       UNIVERSAL::isa($value, 'HASH')
         && CORE::values(%$value) == grep $where->check($_), CORE::values(%$value)
     });
   }
-  elsif (lc($type) eq 'array') {
+  elsif (lc($type) eq 'array' || lc($type) eq 'arrayref') {
     $self->defined(sub{
       my $value = $_->value;
       UNIVERSAL::isa($value, 'ARRAY')
@@ -510,6 +598,14 @@ This package has the following attributes:
 
 =cut
 
+=head2 expects
+
+  expects(ArrayRef)
+
+This attribute is read-write, accepts C<(ArrayRef)> values, and is optional.
+
+=cut
+
 =head2 message
 
   message(Str)
@@ -754,6 +850,33 @@ I<Since C<1.40>>
 
 =cut
 
+=head2 arrayref
+
+  arrayref(CodeRef $check) (Assert)
+
+The arrayref method configures the object to accept array references and
+returns the invocant.
+
+I<Since C<1.71>>
+
+=over 4
+
+=item arrayref example 1
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->arrayref;
+
+  # $assert->check([]);
+
+  # true
+
+=back
+
+=cut
+
 =head2 boolean
 
   boolean(CodeRef $check) (Assert)
@@ -908,6 +1031,33 @@ I<Since C<1.40>>
 
 =cut
 
+=head2 coderef
+
+  coderef(CodeRef $check) (Assert)
+
+The coderef method configures the object to accept code references and returns
+the invocant.
+
+I<Since C<1.71>>
+
+=over 4
+
+=item coderef example 1
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->coderef;
+
+  # $assert->check(sub{});
+
+  # true
+
+=back
+
+=cut
+
 =head2 coerce
 
   coerce(Any $data) (Any)
@@ -1198,6 +1348,122 @@ I<Since C<1.40>>
 
 =cut
 
+=head2 expression
+
+  expression(Str $expr) (Assert)
+
+The expression method parses a string representation of a method/function
+signature, registers the subexpressions using the L</accept> method, and
+returns the invocant.
+
+I<Since C<1.71>>
+
+=over 4
+
+=item expression example 1
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->expression('string');
+
+  # $assert->check('hello');
+
+  # true
+
+  # $assert->check(['goodbye']);
+
+  # false
+
+=back
+
+=over 4
+
+=item expression example 2
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->expression('string | coderef');
+
+  # $assert->check('hello');
+
+  # true
+
+  # $assert->check(sub{'hello'});
+
+  # true
+
+  # $assert->check(['goodbye']);
+
+  # false
+
+=back
+
+=over 4
+
+=item expression example 3
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->expression('string | coderef | Venus::Assert');
+
+  # $assert->check('hello');
+
+  # true
+
+  # $assert->check(sub{'hello'});
+
+  # true
+
+  # $assert->check($assert);
+
+  # true
+
+  # $assert->check(['goodbye']);
+
+  # false
+
+=back
+
+=over 4
+
+=item expression example 4
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->expression('Venus::Assert | arrayref[Venus::Assert]');
+
+  # $assert->check('hello');
+
+  # false
+
+  # $assert->check(sub{'hello'});
+
+  # false
+
+  # $assert->check($assert);
+
+  # true
+
+  # $assert->check(['goodbye']);
+
+  # false
+
+  # $assert->check([$assert]);
+
+  # true
+
+=back
+
+=cut
+
 =head2 float
 
   float(CodeRef $check) (Assert)
@@ -1394,6 +1660,33 @@ I<Since C<1.40>>
 
 =cut
 
+=head2 hashref
+
+  hashref(CodeRef $check) (Assert)
+
+The hashref method configures the object to accept hash references and returns
+the invocant.
+
+I<Since C<1.71>>
+
+=over 4
+
+=item hashref example 1
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->hashref;
+
+  # $assert->check({});
+
+  # true
+
+=back
+
+=cut
+
 =head2 identity
 
   identity(Str $name) (Assert)
@@ -1641,6 +1934,33 @@ I<Since C<1.40>>
 
 =cut
 
+=head2 scalarref
+
+  scalarref(CodeRef $check) (Assert)
+
+The scalarref method configures the object to accept scalar references and returns
+the invocant.
+
+I<Since C<1.71>>
+
+=over 4
+
+=item scalarref example 1
+
+  # given: synopsis
+
+  package main;
+
+  $assert = $assert->scalarref;
+
+  # $assert->check(\1);
+
+  # true
+
+=back
+
+=cut
+
 =head2 string
 
   string(CodeRef $check) (Assert)
